<?php
/**
 * Обработка запроса при добавлении товара в корзину
 * 
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    6.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2018 OOO «Диафан» (http://www.diafan.ru/)
 */

if (! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

class Shop_action extends Action
{
	/**
	 * Добавляет товар в корзину
	 * 
	 * @return void
	 */
	replace public function buy()
	{
		if (empty($_POST['good_id']) || $this->diafan->configmodules('security_user') && ! $this->diafan->_users->id)
		{
			return false;
		}
		if(! $cart_link = $this->diafan->_route->module("cart"))
		{
			return false;
		}

		$count = $this->diafan->filter($_POST, "float", "count", 1);
		if(! $count)
		{
			$this->result["errors"][0] = $this->diafan->_('Количество товара должно быть больше нуля.');
			return;
		}
		$count = $count > 0 ? $count : 1;

		$good_id = $this->diafan->filter($_POST, 'int', 'good_id');
		$this->tag = 'shop'.$good_id;

		$row = DB::query_fetch_array("SELECT id, is_file, [measure_unit] FROM {shop} WHERE id=%d AND trash='0' AND [act]='1' LIMIT 1", $good_id);

		if (empty($row['id']))
		{
			$this->result["errors"][0] = 'ERROR';
			return;
		}

		$params = array();

		$rows_param = DB::query_fetch_all(
				"SELECT p.[name], p.id FROM {shop_param} AS p"
				." INNER JOIN {shop_param_element} AS e ON e.element_id=%d AND e.param_id=p.id"
				." WHERE (p.`type`='multiple' OR p.`type`='color') AND p.required='1' AND p.trash='0' GROUP BY p.id",
				$good_id
			);
		foreach ($rows_param as $row_param)
		{
			if (empty($_POST["param".$row_param["id"]]))
			{
				$this->result["errors"][0] = $this->diafan->_('Пожалуйста, выберите %s.', false, $row_param["name"]);
				return;
			}
			$params[$row_param["id"]] = $this->diafan->filter($_POST, "int", "param".$row_param["id"]);
		}

		$additional_cost_arr = array();
		$additional_cost = '';
		if(! empty($_POST["additional_cost"]))
		{
			$a_cs = array();
			foreach($_POST["additional_cost"] as $a_c)
			{
				$a_c = $this->diafan->filter($a_c, "integer");
				if($a_c)
				{
					$a_cs[] = $a_c;
				}
			}
			if($a_cs)
			{
				$additional_cost_arr = DB::query_fetch_value("SELECT additional_cost_id FROM {shop_additional_cost_rel} WHERE element_id=%d AND trash='0' AND additional_cost_id IN (%s)", $good_id, implode(',', $a_cs), "additional_cost_id");
			}
		}
		
		$additional_cost_arr_2 = DB::query_fetch_value("SELECT r.additional_cost_id FROM {shop_additional_cost_rel} AS r INNER JOIN {shop_additional_cost} AS a ON a.id=r.additional_cost_id WHERE r.element_id=%d AND r.trash='0' AND a.required='1'", $good_id, "additional_cost_id");
		if($additional_cost_arr_2)
		{
			$additional_cost_arr = array_unique(array_merge($additional_cost_arr, $additional_cost_arr_2));
		}

		if($additional_cost_arr)
		{
			sort($additional_cost_arr);
			$additional_cost = implode(',', $additional_cost_arr);
		}

		$count_good = $this->diafan->_cart->get($good_id, $params, $additional_cost, "count");
		$count_good += $count;

		$cart = array(
				"count" => $count_good,
				"is_file" => $row['is_file'],
			);

		if($err = $this->diafan->_cart->set($cart, $good_id, $params, $additional_cost))
		{
			$this->result["errors"][0] = $err;
			return;	
		}
		$this->diafan->_cart->write();

		DB::query("UPDATE {shop} SET counter_buy=counter_buy+1 WHERE id='%d'", $good_id);

		$measure_unit = ! empty($row["measure_unit"]) ? $row["measure_unit"] : $this->diafan->_('шт.');
		$this->result["errors"][0] = $this->diafan->_('В <a href="%s">корзине</a> %s %s', false, BASE_PATH_HREF.$cart_link.'?'.rand(0, 999999), $count_good, $measure_unit);

		Custom::inc('modules/cart/cart.model.php');
		$model = new Cart_model($this->diafan);
		$cart_tpl = $model->show_block();
		$this->result["data"] = array("#show_cart" => $this->diafan->_tpl->get('info', 'cart', $cart_tpl));
		if($this->diafan->_site->module == 'cart')
		{
			$this->result["redirect"] = BASE_PATH_HREF.$this->diafan->_route->current_link().'?'.rand(0, 99999);
		}
	}

}