<?php
/**
 * Модель модуля «Магазин»
 * 
 * @package    DIAFAN.CMS
 * @author     diafan.ru
 * @version    6.0
 * @license    http://www.diafan.ru/license.html
 * @copyright  Copyright (c) 2003-2018 OOO «Диафан» (http://www.diafan.ru/)
 */
if ( ! defined('DIAFAN'))
{
	$path = __FILE__;
	while(! file_exists($path.'/includes/404.php'))
	{
		$parent = dirname($path);
		if($parent == $path) exit;
		$path = $parent;
	}
	include $path.'/includes/404.php';
}

/**
 * Shop_model
 */
class Shop_model extends Model
{

	/**
	 * Генерирует контент для шаблонной функции: форма поиска по товарам
	 *
	 * @param array $site_ids страницы сайта
	 * @param array|string $cat_ids номера категорий
	 * @param string $ajax подгружать результаты поиска Ajax-запросом
	 * @return array
	 */
	replace public function show_search($site_ids, $cat_ids, $ajax)
	{
		//кеширование
		$cache_meta = array(
			"name" => "show_search",
			"lang_id" => _LANG,
			"cat_ids" => $cat_ids,
			"site_ids" => $site_ids,
			"access" => $this->diafan->configmodules('where_access_cat', 'shop') ? $this->diafan->_users->role_id : 0,
		);

		if (! $result = $this->diafan->_cache->get($cache_meta, "shop"))
		{
			if($cat_ids === 'all')
			{
				$cat_ids = array();
				$cat_ids_all = true;
			}
			$one_cat_id = count($cat_ids) == 1 && substr($cat_ids[0], 0, 1) !== '-' ? $cat_ids[0] : 0;
			$minus = array();
			if(! $this->validate_attribute_site_cat('shop', $site_ids, $cat_ids, $minus))
			{
				return false;
			}
			$result["cat_ids"] = array();
			if(count($cat_ids) > 1 || ! empty($cat_ids_all))
			{
				if(empty($cat_ids_all))
				{
					$cats = DB::query_fetch_all("SELECT id, [name], site_id, parent_id FROM {shop_category} WHERE id IN (%s) ORDER BY sort ASC", implode(',', $cat_ids));
				}
				else
				{
					$where = "";
					if($site_ids)
					{
						$where .= " AND c.site_id IN (".implode(',', $site_ids).")";
					}
					elseif(! empty($minus["site_ids"]))
					{
						$where .= " AND c.site_id NOT IN (".implode(',', $minus["site_ids"]).")";
					}
					if(! empty($minus["cat_ids"]))
					{
						$where .= " AND c.id NOT IN (".implode(",", $minus["cat_ids"]).")";
					}
					$cats = DB::query_fetch_all("SELECT c.id, c.[name], c.site_id, c.parent_id FROM {shop_category} AS c"
					.($this->diafan->configmodules('where_access_cat', 'shop') ? " LEFT JOIN {access} AS a ON a.element_id=c.id AND a.module_name='shop' AND a.element_type='cat'" : "")
					." WHERE c.[act]='1' AND c.trash='0'"
					.$where
					.($this->diafan->configmodules('where_access_cat', 'shop') ? " AND (c.access='0' OR c.access='1' AND a.role_id=".$this->diafan->_users->role_id.")" : '')
					." GROUP BY c.id ORDER BY c.sort ASC");
				}
				foreach ($cats as &$cat)
				{
					$cat["level"] = 0;
					$cat_ids[] = $cat["id"];
					$parents[$cat["id"]] = $cat["parent_id"];
				}
				foreach ($cats as &$cat)
				{
					$parent = $cat["id"];
					$level = 0;
					while($parent)
					{
						if(! empty($parents[$parent]))
						{
							$parent = $parents[$parent];
							$level++;
						}
						else
						{
							$parent = 0;
						}
					}
					$cat["level"] = $level;
					$cats_h[$level ? $cat["parent_id"] : 0][] = $cat;
				}
				if($cats)
				{
					$this->list_cats_hierarhy($result["cat_ids"], $cats_h, ($one_cat_id ? $parents[$one_cat_id] : 0));
				}
			}
			elseif(count($cat_ids) == 1)
			{
				$result["cat_ids"][] = array("id" => $cat_ids[0]);
			}
			if(count($site_ids) > 1)
			{
				$result["site_ids"] = DB::query_fetch_all("SELECT id, [name] FROM {site} WHERE id IN (%s) ORDER BY sort ASC", implode(',', $site_ids));
				foreach ($result["site_ids"] as &$site)
				{
					$site["path"] = $this->diafan->_route->link($site["id"]);
				}
			}
			else
			{
				$result["site_ids"][] = array(
						"id" => $site_ids[0],
						"path" => $this->diafan->_route->link($site_ids[0])
					);
			}

			if ($this->diafan->configmodules("search_article", "shop", $site_ids[0]))
			{
				$result["article"] = array(
					"article" => 1,
					"value" => ''
				);
			}
	
			if ($this->diafan->configmodules("search_price", "shop", $site_ids[0]))
			{
				$result["price"] = array(
					"name" => 1,
					"value1" => 0,
					"value2" => 0
				);
			}
	
			if ($this->diafan->configmodules("search_brand", "shop", $site_ids[0]))
			{
				$result["brands"] = DB::query_fetch_all("SELECT b.id, b.[name], b.site_id FROM {shop_brand} AS b"
					.($cat_ids ? " INNER JOIN {shop_brand_category_rel} AS r ON r.element_id=b.id" : "")
					." WHERE b.[act]='1' AND b.trash='0' AND b.site_id IN (%s)"
					." AND (SELECT s.id FROM {shop} AS s ".($cat_ids ? " INNER JOIN {shop_category_rel} AS cs ON cs.element_id=s.id" : '')." WHERE s.brand_id=b.id AND s.[act]='1' AND s.trash='0' ".($cat_ids ? " AND cs.cat_id IN (%s)" : '')." LIMIT 1)"
					.($cat_ids ? " AND r.cat_id IN (0,%s) GROUP BY b.id" : "")
					." ORDER BY b.sort ASC",
					implode(',', $site_ids),
					implode(',', $cat_ids),
					implode(',', $cat_ids));
				$result["brand"] = array();
			}
	
			if ($this->diafan->configmodules("search_action", "shop", $site_ids[0]))
			{
				$result["action"] = array(
					"name" => 1,
					"value" => false
				);
			}
	
			if ($this->diafan->configmodules("search_hit", "shop", $site_ids[0]))
			{
				$result["hit"] = array(
					"name" => 1,
					"value" => false
				);
			}
	
			if ($this->diafan->configmodules("search_new", "shop", $site_ids[0]))
			{
				$result["new"] = array(
					"name" => 1,
					"value" => false
				);
			}
	
			$result["rows"] = DB::query_fetch_all("SELECT p.id, p.type, p.[name], p.[measure_unit], GROUP_CONCAT(c.cat_id SEPARATOR ',') as cat_ids FROM {shop_param} as p "
				." INNER JOIN {shop_param_category_rel} AS c ON p.id=c.element_id AND "
				.($cat_ids ? "(c.cat_id IN (".implode(',', $cat_ids).") OR c.cat_id=0)" : "c.cat_id=0")
				." WHERE p.search='1' AND p.trash='0'"
				.($site_ids ? " AND p.site_id IN (0, ".implode(",", $site_ids).")" : '')
				." GROUP BY p.id ORDER BY p.sort ASC");
	
			foreach ($result["rows"] as $i => &$row)
			{
				if(! isset($row["cat_ids"]))
				{
					$row["cat_ids"] = '';
				}
				if ($row["type"] == 'select' || $row["type"] == 'multiple' || $row["type"] == 'color')
				{
					$row["select_array"] = DB::query_fetch_key_value(
						"SELECT p.[name], p.id, p.path FROM {shop_param_select} AS p"
						// выводим значения, только есть товары, чтобы поиск не давал пустых результатов
						." INNER JOIN {shop_param_element} AS e ON p.param_id=e.param_id AND e.value".$this->diafan->_languages->site."=p.id"
						." INNER JOIN {shop} AS s ON e.element_id=s.id AND s.[act]='1' AND s.trash='0'"
						.($this->diafan->configmodules('hide_missing_goods') ? " AND s.no_buy='0'" : "")
						.($this->diafan->configmodules('hide_missing_goods') && $this->diafan->configmodules('use_count_goods') ? " INNER JOIN {shop_price} AS prh ON prh.good_id=s.id AND prh.count_goods>0" : "")
						.($cat_ids ? " INNER JOIN {shop_category_rel} AS c ON c.element_id=s.id AND c.cat_id IN (".implode(",", $cat_ids).")" : '')
						." WHERE p.param_id=%d GROUP BY p.id ORDER BY p.sort ASC", $row["id"], "id", "name");
					if(empty($row["select_array"]))
					{
						unset($result["rows"][$i]);
					}
				}
			}
		}

		if (! empty($result["article"]))
		{
			$result["article"]["value"] = ! empty($_REQUEST["a"]) ? trim(htmlspecialchars(stripslashes($_REQUEST["a"]))) : '';
		}

		if (! empty($result["price"]))
		{
			$result["price"]["value1"] = $this->diafan->filter($_REQUEST, "int", "pr1");
			$result["price"]["value2"] = $this->diafan->filter($_REQUEST, "int", "pr2");
		}

		if (! empty($result["brands"]))
		{
			if(! empty($_REQUEST["brand"]))
			{
				if(is_array($_REQUEST["brand"]))
				{
					foreach($_REQUEST["brand"] as $b)
					{
						$b = $this->diafan->filter($b, "integer");
						if($b)
						{
							$result["brand"][] = $b;
						}
					}
				}
				else
				{
					$b = $this->diafan->filter($_REQUEST, "integer", "brand");
					if($b)
					{
						$result["brand"][] = $b;
					}
				}
			}
			elseif($this->diafan->_route->brand)
			{
				$result["brand"][] = $this->diafan->_route->brand;
			}
		}

		if (! empty($result["action"]) && ! empty($_REQUEST["ac"]))
		{
			$result["action"]["value"] = true;
		}

		if (! empty($result["hit"]) && ! empty($_REQUEST["hi"]))
		{
			$result["hit"]["value"] = true;
		}

		if (! empty($result["new"]) && ! empty($_REQUEST["ne"]))
		{
			$result["new"]["value"] = true;
		}
		
		foreach ($result["rows"] as &$row)
		{
			switch($row["type"])
			{
				case 'date':
				case 'datetime':
					$row["value1"] = $this->diafan->filter($_REQUEST, "string", "p".$row["id"]."_1");
					$row["value2"] = $this->diafan->filter($_REQUEST, "string", "p".$row["id"]."_2");
					break;

				case 'numtext':
					$row["value1"] = $this->diafan->filter($_REQUEST, "float", "p".$row["id"]."_1");
					$row["value2"] = $this->diafan->filter($_REQUEST, "float", "p".$row["id"]."_2");
					break;

				case 'checkbox':
					$row["value"] = $this->diafan->filter($_REQUEST, "int", "p".$row["id"]);
					break;

				case 'select':
				case 'multiple':
				case 'color':
					$row["value"] = array();
					if ( ! empty($_REQUEST["p".$row["id"]]) && ! is_array($_REQUEST["p".$row["id"]]))
					{
						$row["value"][] = $this->diafan->filter($_REQUEST, "int", "p".$row["id"]);
					}
					elseif ( ! empty($_REQUEST["p".$row["id"]]) && is_array($_REQUEST["p".$row["id"]]))
					{
						foreach ($_REQUEST["p".$row["id"]] as $val)
						{
							$row["value"][] = intval($val);
						}
					}
					break;

				default:
					$row["value"] = array();
			}
		}

		if($this->diafan->_site->module == 'shop' && in_array($this->diafan->_site->id, $site_ids))
		{
			$result["site_id"] = $this->diafan->_site->id;
			foreach ($result["site_ids"] as &$row)
			{
				if($row["id"] == $this->diafan->_site->id)
				{
					$result["path"] = $row["path"];
				}
			}
		}
		else
		{
			$result["site_id"] = $result["site_ids"][0]["id"];
			$result["path"]    = $result["site_ids"][0]["path"];
		}
		if($this->diafan->_site->module == 'shop' && in_array($this->diafan->_route->cat, $cat_ids))
		{
			$result["cat_id"] = $this->diafan->_route->cat;
		}
		elseif(! empty($result["cat_ids"][0]["id"]) && count($result["cat_ids"]) == 1)
		{
			$result["cat_id"] = $result["cat_ids"][0]["id"];
		}
		else
		{
			$result["cat_id"] = 0;
		}
		$result["send_ajax"] = $ajax;
		return $result;
	}


	/**
	 * Формирует данные о цене товара
	 *
	 * @param array $row данные о товаре
	 * @return void
	 */
	replace private function price(&$row)
	{
		// массив всех характеристик, доступных к выбору при заказе
		if (! isset($this->result["depends_param"]))
		{
			$this->result["depends_param"] = DB::query_fetch_all("SELECT id, [name] FROM {shop_param} WHERE (`type`='multiple' OR `type`='color') AND required='1' AND trash='0' ORDER BY sort ASC");
			foreach ($this->result["depends_param"] as &$row_param)
			{
				$row_param["values"] = DB::query_fetch_all("SELECT id, [name], path FROM {shop_param_select} WHERE param_id=%d AND trash='0' ORDER BY sort ASC", $row_param["id"]);
			}
		}

		$price = array();
		$count = 0;
		$row["param_multiple"] = array();
		$rows = $this->diafan->_shop->price_get_all($row["id"], $this->diafan->_users->id);
		foreach ($rows as $row_price)
		{
			$empty_param = array();
			$row_price["param"] = array();
			$rows_param = DB::query_fetch_all("SELECT param_id, param_value FROM {shop_price_param} WHERE price_id=%d", $row_price["price_id"]);
			foreach ($rows_param as &$row_param)
			{
				if(! empty($row_param["param_value"]))
				{
					$row["param_multiple"][$row_param["param_id"]][$row_param["param_value"]] = 'depend';
					$row_price["param"][] = array("id" => $row_param["param_id"], "value" => $row_param["param_value"]);
				}
				else
				{
					$empty_param[] = $row_param["param_id"];
				}
			}
			if(! empty($empty_param))
			{
				$rows_param = DB::query_fetch_all("SELECT param_id, value".$this->diafan->_languages->site." AS value FROM {shop_param_element} WHERE element_id=%d AND param_id IN (%s)", $row["id"], implode(",", $empty_param));
				foreach ($rows_param as $row_param)
				{
					$row["param_multiple"][$row_param["param_id"]][$row_param["value"]] = 'select';
				}
			}
			$count += $row_price["count_goods"];
			$row_price["count"] = $this->diafan->configmodules("use_count_goods", "shop") ? $row_price["count_goods"] : true;
			$row_price["image_rel"] = DB::query_result("SELECT image_id FROM {shop_price_image_rel} WHERE price_id=%d LIMIT 1", $row_price["price_id"]);
			$price[] = $row_price;
		}
		$row["price_arr"] = $price;
		$row["count"] = $this->diafan->configmodules("use_count_goods", "shop") ? $count : true;
		$row["price"] = !empty($price[0]) ? $price[0]["price"] : 0;
		$row["old_price"] = !empty($price[0]) ? $price[0]["old_price"] : 0;
		$row["discount"] = '';
		if(! empty($price[0]) && $price[0]["discount_id"])
		{
			if($price[0]["discount"])
			{
				$row["discount"] = $price[0]["discount"];
				$row["discount_currency"] = '%';
			}
			else
			{
				$row["discount"] = $this->diafan->_shop->price_format($price[0]["old_price"] - $price[0]["price"]);
				$row["discount_currency"] = $this->diafan->configmodules("currency", "shop");
			}
			$row["discount_finish"] = !empty($price[0]["date_finish"]) ? $this->format_date($price[0]["date_finish"], "shop") : '';
		}
	}

	/**
	 * Получает дополнительные характеристики товара
	 * 
	 * @param integer $id номер товара
	 * @param integer $site_id номер страницы, к которой прикреплен товар
	 * @param string $function функция, для которой выбираются параметры
	 * @return array
	 */
	replace private function param(&$good, $function = "id")
	{
		global $param_select, $param_select_page;
		$values = DB::query_fetch_key_array("SELECT e.value".$this->diafan->_languages->site." as rvalue, e.[value], e.param_id, e.id, s.path as spath FROM {shop_param_element} as e"
		. " LEFT JOIN {shop_param_select} as s ON e.param_id=s.param_id AND e.value".$this->diafan->_languages->site."=s.id"
		. " WHERE e.element_id=%d GROUP BY e.id ORDER BY s.sort ASC", $good["id"], "param_id");

		$rows = DB::query_fetch_all("SELECT p.id, p.[name], p.type, p.page, p.[measure_unit], p.config, p.[text], p.block, p.list, p.id_page FROM {shop_param} as p "
		. ($this->diafan->configmodules("cat", "shop", $good["site_id"]) ? " INNER JOIN {shop_category_rel} as c ON c.element_id=".$good["id"] : "")
		. " INNER JOIN {shop_param_category_rel} as cp ON cp.element_id=p.id "
		. ($this->diafan->configmodules("cat", "shop", $good["site_id"]) ?
				" AND (cp.cat_id=c.cat_id OR cp.cat_id=0) " : "")
		. " WHERE p.trash='0' "
		. " GROUP BY p.id ORDER BY p.sort ASC"
		);

		$good["param"] = array();
		$good["all_param"] = array();
		$colorpath = array();
		foreach ($rows as $row)
		{
			switch ($row["type"])
			{
				case "text":
				case "textarea":
				case "editor":
					if ( ! empty($values[$row["id"]][0]["value"]))
					{
						$row["value"] = $values[$row["id"]][0]["value"];
						$row["value_id"] = $values[$row["id"]][0]["id"];
					}
					break;

				case "date":
					if ( ! empty($values[$row["id"]][0]["rvalue"]))
					{
						$row["value"] = $this->diafan->formate_from_date($values[$row["id"]][0]["rvalue"]);
						$row["value_id"] = $values[$row["id"]][0]["id"];
					}
					break;

				case "datetime":
					if ( ! empty($values[$row["id"]][0]["rvalue"]))
					{
						$row["value"] = $this->diafan->formate_from_datetime($values[$row["id"]][0]["rvalue"]);
						$row["value_id"] = $values[$row["id"]][0]["id"];
					}
					break;

				case "select":
					$value = ! empty($values[$row["id"]][0]["rvalue"]) ? $values[$row["id"]][0]["rvalue"] : '';
					if ($value)
					{
						if (empty($this->cache["param_select"][$row["id"]][$value]))
						{
							$this->cache["param_select"][$row["id"]][$value] = DB::query_result("SELECT [name] FROM {shop_param_select} WHERE id=%d AND param_id=%d LIMIT 1", $values[$row["id"]][0]["rvalue"], $row["id"]);
						}
						if ($row["page"])
						{
							if (empty($this->cache["param_select_page"][$row["id"]][$value]))
							{
								$this->cache["param_select_page"][$row["id"]][$value] = $this->diafan->_route->link($good["site_id"], $value, "shop", 'param');
							}
							$row["link"] = $this->cache["param_select_page"][$row["id"]][$value];
						}
						$row["value"] = $this->cache["param_select"][$row["id"]][$value];
					}
					break;

				case "multiple":
				case "color":
				if ( ! empty($values[$row["id"]]))
				{
					$value = array();
					$spath = array();
					foreach ($values[$row["id"]] as $val)
					{
						if (empty($this->cache["param_select"][$row["id"]][$val["rvalue"]]))
						{
							$this->cache["param_select"][$row["id"]][$val["rvalue"]] =
								DB::query_result("SELECT [name] FROM {shop_param_select} WHERE id=%d AND param_id=%d LIMIT 1", $val["rvalue"], $row["id"]);
						}
						if ($row["page"])
						{
							if ($this->diafan->_site->module == 'shop' && $this->diafan->_route->param == $val["rvalue"])
							{
								$link = '';
							}
							else
							{
								if (empty($this->cache["param_select_page"][$row["id"]][$val["rvalue"]]))
								{
									$this->cache["param_select_page"][$row["id"]][$val["rvalue"]] = $this->diafan->_route->link($good["site_id"], $val["rvalue"], "shop", 'param');
								}
								$link = $this->cache["param_select_page"][$row["id"]][$val["rvalue"]];
							}
							$value[] = array("id" => $val["rvalue"], "name" => $this->cache["param_select"][$row["id"]][$val["rvalue"]], "link" => $link);
						}
						else
						{
							$value[] = $this->cache["param_select"][$row["id"]][$val["rvalue"]];
						}
						$spath[] = $val["spath"];
					}
					$row["value"] = $value;
					$colorpath = $spath;
				}
					break;

				case "checkbox":
					$value = ! empty($values[$row["id"]][0]["rvalue"]) ? 1 : 0;
					if ( ! isset($this->cache["param_select"][$row["id"]][$value]))
					{
						$this->cache["param_select"][$row["id"]][$value] =
								DB::query_result("SELECT [name] FROM {shop_param_select} WHERE value=%d AND param_id=%d LIMIT 1", $value, $row["id"]);
					}
					if ( ! $this->cache["param_select"][$row["id"]][$value])
					{
						if($value == 1)
						{
							$row["value"] = '';
						}
					}
					else
					{
						$row["value"] = $this->cache["param_select"][$row["id"]][$value];
					}
					break;

				case "title":
					$row["value"] = '';
					break;

				case "images":
					$value = $this->diafan->_images->get('large', $good["id"], "shop", 'element', 0, '', $row["id"]);
					if(! $value)
						continue 2;

					$row["value"] = $value;
					break;

				case "attachments":
					$config = unserialize($row["config"]);
					if($config["attachments_access_admin"])
						continue 2;

					$value = $this->diafan->_attachments->get($good["id"], "shop", $row["id"]);
					if(! $value)
						continue 2;

					$row["value"] = $value;
					$row["use_animation"] = ! empty($config["use_animation"]) ? true : false;
					break;

				default:
					if ( ! empty($values[$row["id"]][0]["rvalue"]))
					{
						$row["value"] = $values[$row["id"]][0]["rvalue"];
						$row["value_id"] = $values[$row["id"]][0]["id"];
					}
					break;
			}
			if(isset($row["value"]))
			{
				$param = array(
					"id" => $row["id"],
					"name" => $row["name"],
					"value" => $row["value"],
					"value_id" => (! empty($row["value_id"]) ? $row["value_id"] : ''),
					"use_animation" => ! empty($row["use_animation"]) ? true : false,
					"text" => $row["text"],
					"type" => $row["type"],
					"measure_unit" => $row["measure_unit"],
					"link" => (! empty($row["link"]) ? $row["link"] : ''),
					"colorpath" => $colorpath
				);
				$good["all_param"][] = $param;
				switch($function)
				{
					case "block":
						if($row["block"])
						{
							$good["param"][] = $param;
						}
						break;

					case "list":
						if($row["list"])
						{
							$good["param"][] = $param;
						}
						break;

					case "id":
						if($row["id_page"])
						{
							$good["param"][] = $param;
						}
						break;
				}
			}
		}
	}

	/**
	 * Формирует SQL-запрос при поиске по товарам
	 * 
	 * @return boolean true
	 */
	replace private function where(&$where, &$where_param, &$values, &$getnav, &$group)
	{
		$where = ' AND s.site_id=%d';
		$values[] = $this->diafan->_site->id;
		$values_param = array();

		$getnav = '?action=search';
		if (!empty($_REQUEST["cat_id"]))
		{
			$this->diafan->_route->cat = (int) preg_replace("/\D/", '', $_REQUEST['cat_id']);
			$catarr = array(0);
			$getnav .='&cat_id='.$this->diafan->_route->cat;
			if ($this->diafan->_route->cat)
			{
				$children = $this->diafan->get_children($this->diafan->_route->cat, "shop_category");
				$children[] = $this->diafan->_route->cat;
				$where_param .= " INNER JOIN {shop_category_rel} AS c ON s.id=c.element_id AND c.cat_id IN (".implode(',', $children).")";
			}
		}

		if (!empty($_REQUEST["a"]) && $this->diafan->configmodules("search_article"))
		{
			$where .= " AND LOWER(REPLACE(REPLACE(s.article, ' ', ''), '-', ''))='%h'";
			$_REQUEST["a"] = $this->diafan->filter($_REQUEST, "string", "a");
			$values[] = str_replace(array(' ', '-'), '', $_REQUEST["a"]);
			$getnav .= '&a='.$_REQUEST["a"];
		}
	
		if (!empty($_REQUEST["brand"]) && $this->diafan->configmodules("search_brand"))
		{
			$brand = array();
			if(is_array($_REQUEST["brand"]))
			{
				foreach($_REQUEST["brand"] as $b)
				{
					$b = $this->diafan->filter($b, "integer");
					if($b)
					{
						$brand[] = $b;
					}
				}
			}
			else
			{
				$b = $this->diafan->filter($_REQUEST, "integer", "brand");
				if($b)
				{
					$brand[] = $b;
				}
			}
			if($brand)
			{
				$where .= " AND s.brand_id".(count($brand) == 1 ? '='.$brand[0] : ' IN ('.implode(',', $brand).')');
				$getnav .= '&brand[]='.implode('&brand[]=', $brand);
			}
		}
	
		if (!empty($_REQUEST["ac"]) && $this->diafan->configmodules("search_action"))
		{
			$where .= " AND s.action='1'";
			$getnav .= '&ac=1';
		}
	
		if (!empty($_REQUEST["hi"]) && $this->diafan->configmodules("search_hit"))
		{
			$where .= " AND s.hit='1'";
			$getnav .= '&hi=1';
		}
	
		if (!empty($_REQUEST["ne"]) && $this->diafan->configmodules("search_new"))
		{
			$where .= " AND s.new='1'";
			$getnav .= '&ne=1';
		}
	
		if (!empty($_REQUEST["pr1"]) || !empty($_REQUEST["pr2"]))
		{
			if (!empty($_REQUEST["pr1"]))
			{
				$pr1 = $this->diafan->filter($_REQUEST, "int", "pr1");
				$getnav .= '&pr1='.$pr1;
			}
			if (!empty($_REQUEST["pr2"]))
			{
				$pr2 = $this->diafan->filter($_REQUEST, "int", "pr2");
				$getnav .= '&pr2='.$pr2;
			}
			$where_param .= " INNER JOIN {shop_price} AS pr ON pr.good_id=s.id AND pr.trash='0'"
				." AND pr.date_start<=".time()." AND (pr.date_start=0 OR pr.date_finish>=".time().")"
				." AND pr.currency_id=0"
				." AND pr.role_id".($this->diafan->_users->role_id ? " IN (0,".$this->diafan->_users->role_id.")" : "=0")
				." AND (pr.person='0'".($this->person_discount_ids ? " OR pr.discount_id IN(".implode(",", $this->person_discount_ids).")" : "").")";
			$group = ", pr.price_id HAVING"
				.(!empty($_REQUEST["pr1"]) ? " MIN(ROUND(pr.price))>=".$pr1 : '')
				.(!empty($_REQUEST["pr2"]) ? (!empty($_REQUEST["pr1"]) ? " AND" : "")." MIN(pr.price)<=".$pr2 : '');
		}
		else
		{
			$where_param .= " LEFT JOIN {shop_price} AS pr ON pr.good_id=s.id AND pr.trash='0'"
				." AND pr.date_start<=".time()." AND (pr.date_start=0 OR pr.date_finish>=".time().")"
				." AND pr.currency_id=0"
				." AND pr.role_id".($this->diafan->_users->role_id ? " IN (0,".$this->diafan->_users->role_id.")" : "=0")
				." AND (pr.person='0'".($this->person_discount_ids ? " OR pr.discount_id IN(".implode(",", $this->person_discount_ids).")" : "").")";
		}
		$rows = DB::query_fetch_all("SELECT p.id, p.type, p.required FROM {shop_param} as p "
				." INNER JOIN {shop_param_category_rel} AS c ON p.id=c.element_id "
				.($this->diafan->configmodules("cat") ? " AND (c.cat_id=%d OR c.cat_id=0)" : "")
				." WHERE p.search='1' AND p.trash='0' GROUP BY p.id ORDER BY p.sort ASC", $this->diafan->_route->cat);
		foreach ($rows as $row)
		{
			if ($row["type"] == 'date' && (!empty($_REQUEST["p".$row["id"]."_1"]) || !empty($_REQUEST["p".$row["id"]."_2"])))
			{
				$where_param .= " INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0'";
				$values_param[] = $row["id"];
				if (!empty($_REQUEST["p".$row["id"]."_1"]))
				{
					$where_param .= " AND pe".$row["id"].".value".$this->diafan->_languages->site.">='%s'";
					$values_param[] = $this->diafan->formate_in_date($_REQUEST["p".$row["id"]."_1"]);
					$getnav .= '&p'.$row["id"].'_1='.$this->diafan->filter($_REQUEST, "url", "p".$row["id"]."_1");
				}
				if (!empty($_REQUEST["p".$row["id"]."_2"]))
				{
					$where_param .= " AND pe".$row["id"].".value".$this->diafan->_languages->site."<='%s'";
					$values_param[] = $this->diafan->formate_in_date($_REQUEST["p".$row["id"]."_2"]);
					$getnav .= '&p'.$row["id"].'_2='.$this->diafan->filter($_REQUEST, "url", "p".$row["id"]."_2");
				}
			}
			elseif ($row["type"] == 'datetime' && (!empty($_REQUEST["p".$row["id"]."_1"]) || !empty($_REQUEST["p".$row["id"]."_2"])))
			{
				$where_param .= " INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0'";
				$values_param[] = $row["id"];
				if (!empty($_REQUEST["p".$row["id"]."_1"]))
				{
					$where_param .= " AND pe".$row["id"].".value".$this->diafan->_languages->site.">='%s'";
					$values_param[] = $this->diafan->formate_in_datetime($_REQUEST["p".$row["id"]."_1"]);
					$getnav .= '&p'.$row["id"].'_1='.$this->diafan->filter($_REQUEST, "url", "p".$row["id"]."_1");
				}
				if (!empty($_REQUEST["p".$row["id"]."_2"]))
				{
					$where_param .= " AND pe".$row["id"].".value".$this->diafan->_languages->site."<='%s'";
					$values_param[] = $this->diafan->formate_in_datetime($_REQUEST["p".$row["id"]."_2"]);
					$getnav .= '&p'.$row["id"].'_2='.$this->diafan->filter($_REQUEST, "url", "p".$row["id"]."_2");
				}
			}
			elseif ($row["type"] == 'numtext' && (!empty($_REQUEST["p".$row["id"]."_2"]) || !empty($_REQUEST["p".$row["id"]."_1"])))
			{
				$val1 = $this->diafan->filter($_REQUEST, "float", "p".$row["id"]."_1");
				$val2 = $this->diafan->filter($_REQUEST, "float", "p".$row["id"]."_2");
				$where_param .= " INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0'"
					.($val1 ? " AND pe".$row["id"].".value".$this->diafan->_languages->site.">=%f" : '')
					.($val2 ? " AND pe".$row["id"].".value".$this->diafan->_languages->site."<=%f" : '')
				;
				$values_param[] = $row["id"];
				if ($val1)
				{
					$values_param[] = $val1;
					$getnav .= '&p'.$row["id"].'_1='.$val1;
				}
				if ($val2)
				{
					$values_param[] = $val2;
					$getnav .= '&p'.$row["id"].'_2='.$val2;
				}
			}
			elseif ($row["type"] == 'checkbox' && !empty($_REQUEST["p".$row["id"]]))
			{
				$where_param .= " INNER JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".value".$this->diafan->_languages->site."='1'";
				$values_param[] = $row["id"];
				$getnav .= '&p'.$row["id"].'=1';
			}
			elseif (($row["type"] == 'select' || $row["type"] == 'multiple' || $row["type"] == 'color') && !empty($_REQUEST["p".$row["id"]]))
			{
				if (!is_array($_REQUEST["p".$row["id"]]))
				{
					$val = $this->diafan->filter($_REQUEST, "int", "p".$row["id"]);
					$getnav .= '&p'.$row["id"].'='.$val;
					$vals = array($val);
				}
				else
				{
					$vals = array();
					foreach ($_REQUEST["p".$row["id"]] as $val)
					{
						if ($val)
						{
							$val = intval($val);
							$vals[] = $val;
							$getnav .= '&p'.$row["id"].'[]='.$val;
						}
					}
				}
				if (!empty($vals))
				{
					if ($row["required"])
					{
						$where_param .= " INNER JOIN {shop_price_param} AS prp".$row["id"]." ON prp".$row["id"].".price_id=pr.price_id";
						$where .= " AND prp".$row["id"].".param_id=".$row["id"]." AND prp".$row["id"].".param_value IN (".implode(", ", $vals).",0) AND pe".$row["id"].".param_id=".$row["id"];
						if(empty($first_required_param))
						{
							$first_required_param = " AND prp".$row["id"].".price_id=";
						}
						else
						{
							$where .= $first_required_param."prp".$row["id"].".price_id";
						}
					}
					$where_param .= " ".($row["required"] ? "LEFT" : "INNER")." JOIN {shop_param_element} AS pe".$row["id"]." ON pe".$row["id"].".element_id=s.id AND pe".$row["id"].".param_id='%d'"
					." AND pe".$row["id"].".trash='0' AND pe".$row["id"].".value".$this->diafan->_languages->site." IN (".implode(", ", $vals).")";
					$values_param[] = $row["id"];
				}
			}
		}
	
		$values = array_merge($values_param, $values);
	}

}
